@@ -1,24 +1,24 @@
-import pytest
-
-import sys
-import ssl
-import socket
 import datetime
+import socket
+import ssl
+import sys
 from concurrent.futures import ThreadPoolExecutor
-from ipaddress import IPv4Address, IPv6Address, IPv4Network, IPv6Network
+from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
 from pathlib import Path
 from typing import Callable, Optional, Union
 
+import OpenSSL.SSL
+import pytest
+import service_identity.pyopenssl  # type: ignore[import-not-found]
 from cryptography import x509
 from cryptography.hazmat.primitives.serialization import (
-    Encoding, PublicFormat, load_pem_private_key)
-
-import OpenSSL.SSL
-import service_identity.pyopenssl  # type: ignore[import]
+    Encoding,
+    PublicFormat,
+    load_pem_private_key,
+)
 
 import trustme
-from trustme import CA, LeafCert, KeyType
-
+from trustme import CA, KeyType, LeafCert
 
 SslSocket = Union[ssl.SSLSocket, OpenSSL.SSL.Connection]
 
@@ -55,11 +55,13 @@ def assert_is_leaf(leaf_cert: x509.Certificate) -> None:
     assert ku.critical is True
 
     eku = leaf_cert.extensions.get_extension_for_class(x509.ExtendedKeyUsage)
-    assert eku.value == x509.ExtendedKeyUsage([
-        x509.oid.ExtendedKeyUsageOID.CLIENT_AUTH,
-        x509.oid.ExtendedKeyUsageOID.SERVER_AUTH,
-        x509.oid.ExtendedKeyUsageOID.CODE_SIGNING
-    ])
+    assert eku.value == x509.ExtendedKeyUsage(
+        [
+            x509.oid.ExtendedKeyUsageOID.CLIENT_AUTH,
+            x509.oid.ExtendedKeyUsageOID.SERVER_AUTH,
+            x509.oid.ExtendedKeyUsageOID.CODE_SIGNING,
+        ]
+    )
     assert eku.critical is True
 
 
@@ -69,7 +71,7 @@ def assert_is_leaf(leaf_cert: x509.Certificate) -> None:
 def test_basics(key_type: KeyType, expected_key_header: bytes) -> None:
     ca = CA(key_type=key_type)
 
-    today = datetime.datetime.today()
+    today = datetime.datetime.now(datetime.timezone.utc)
 
     assert (
         b"BEGIN " + expected_key_header + b" PRIVATE KEY" in ca.private_key_pem.bytes()
@@ -79,7 +81,7 @@ def test_basics(key_type: KeyType, expected_key_header: bytes) -> None:
     private_key = load_pem_private_key(ca.private_key_pem.bytes(), password=None)
 
     ca_cert = x509.load_pem_x509_certificate(ca.cert_pem.bytes())
-    assert ca_cert.not_valid_before <= today <= ca_cert.not_valid_after
+    assert ca_cert.not_valid_before_utc <= today <= ca_cert.not_valid_after_utc
 
     public_key1 = private_key.public_key().public_bytes(
         Encoding.PEM, PublicFormat.SubjectPublicKeyInfo
@@ -102,13 +104,15 @@ def test_basics(key_type: KeyType, expected_key_header: bytes) -> None:
     assert b"PRIVATE KEY" in server.private_key_pem.bytes()
     assert b"BEGIN CERTIFICATE" in server.cert_chain_pems[0].bytes()
     assert len(server.cert_chain_pems) == 1
-    assert server.private_key_pem.bytes() in server.private_key_and_cert_chain_pem.bytes()
+    assert (
+        server.private_key_pem.bytes() in server.private_key_and_cert_chain_pem.bytes()
+    )
     for blob in server.cert_chain_pems:
         assert blob.bytes() in server.private_key_and_cert_chain_pem.bytes()
 
     server_cert = x509.load_pem_x509_certificate(server.cert_chain_pems[0].bytes())
 
-    assert server_cert.not_valid_before <= today <= server_cert.not_valid_after
+    assert server_cert.not_valid_before_utc <= today <= server_cert.not_valid_after_utc
     assert server_cert.issuer == ca_cert.subject
     assert_is_leaf(server_cert)
 
@@ -119,38 +123,32 @@ def test_basics(key_type: KeyType, expected_key_header: bytes) -> None:
 
 def test_ca_custom_names() -> None:
     ca = CA(
-        organization_name='python-trio',
-        organization_unit_name='trustme',
+        organization_name="python-trio",
+        organization_unit_name="trustme",
     )
 
     ca_cert = x509.load_pem_x509_certificate(ca.cert_pem.bytes())
 
     assert {
-        'O=python-trio',
-        'OU=trustme',
-    }.issubset({
-        rdn.rfc4514_string()
-        for rdn in ca_cert.subject.rdns
-    })
+        "O=python-trio",
+        "OU=trustme",
+    }.issubset({rdn.rfc4514_string() for rdn in ca_cert.subject.rdns})
 
 
 def test_issue_cert_custom_names() -> None:
     ca = CA()
     leaf_cert = ca.issue_cert(
-        'example.org',
-        organization_name='python-trio',
-        organization_unit_name='trustme',
+        "example.org",
+        organization_name="python-trio",
+        organization_unit_name="trustme",
     )
 
     cert = x509.load_pem_x509_certificate(leaf_cert.cert_chain_pems[0].bytes())
 
     assert {
-        'O=python-trio',
-        'OU=trustme',
-    }.issubset({
-        rdn.rfc4514_string()
-        for rdn in cert.subject.rdns
-    })
+        "O=python-trio",
+        "OU=trustme",
+    }.issubset({rdn.rfc4514_string() for rdn in cert.subject.rdns})
 
 
 def test_issue_cert_custom_not_after() -> None:
@@ -168,7 +166,24 @@ def test_issue_cert_custom_not_after() -> None:
     cert = x509.load_pem_x509_certificate(leaf_cert.cert_chain_pems[0].bytes())
 
     for t in ["year", "month", "day", "hour", "minute", "second"]:
-        assert getattr(cert.not_valid_after, t) == getattr(expires, t)
+        assert getattr(cert.not_valid_after_utc, t) == getattr(expires, t)
+
+
+def test_issue_cert_custom_not_before() -> None:
+    not_before = datetime.datetime(2027, 7, 5, 17, 15, 30)
+    ca = CA()
+
+    leaf_cert = ca.issue_cert(
+        "example.org",
+        organization_name="python-trio",
+        organization_unit_name="trustme",
+        not_before=not_before,
+    )
+
+    cert = x509.load_pem_x509_certificate(leaf_cert.cert_chain_pems[0].bytes())
+
+    for t in ["year", "month", "day", "hour", "minute", "second"]:
+        assert getattr(cert.not_valid_before_utc, t) == getattr(not_before, t)
 
 
 def test_intermediate() -> None:
@@ -183,6 +198,12 @@ def test_intermediate() -> None:
     assert_is_ca(child_ca_cert)
     assert child_ca_cert.issuer == ca_cert.subject
     assert _path_length(child_ca_cert) == 8
+    aki = child_ca_cert.extensions.get_extension_for_class(x509.AuthorityKeyIdentifier)
+    assert aki.critical is False
+    expected_aki_key_id = ca_cert.extensions.get_extension_for_class(
+        x509.SubjectKeyIdentifier
+    ).value.digest
+    assert aki.value.key_identifier == expected_aki_key_id
 
     child_server = child_ca.issue_cert("test-host.example.org")
     assert len(child_server.cert_chain_pems) == 2
@@ -255,6 +276,7 @@ def test_blob(tmp_path: Path) -> None:
         with open(path, "rb") as f:
             assert f.read() == test_data
 
+
 def test_ca_from_pem(tmp_path: Path) -> None:
     ca1 = trustme.CA()
     ca2 = trustme.CA.from_pem(ca1.cert_pem.bytes(), ca1.private_key_pem.bytes())
@@ -337,11 +359,12 @@ def check_connection_end_to_end(
 
 @pytest.mark.parametrize("key_type", [KeyType.RSA, KeyType.ECDSA])
 def test_stdlib_end_to_end(key_type: KeyType) -> None:
-    def wrap_client(ca: CA, raw_client_sock: socket.socket, hostname: str) -> ssl.SSLSocket:
+    def wrap_client(
+        ca: CA, raw_client_sock: socket.socket, hostname: str
+    ) -> ssl.SSLSocket:
         ctx = ssl.create_default_context()
         ca.configure_trust(ctx)
-        wrapped_client_sock = ctx.wrap_socket(
-            raw_client_sock, server_hostname=hostname)
+        wrapped_client_sock = ctx.wrap_socket(raw_client_sock, server_hostname=hostname)
         print("Client got server cert:", wrapped_client_sock.getpeercert())
         peercert = wrapped_client_sock.getpeercert()
         assert peercert is not None
@@ -349,11 +372,12 @@ def test_stdlib_end_to_end(key_type: KeyType) -> None:
         assert san == (("DNS", "my-test-host.example.org"),)
         return wrapped_client_sock
 
-    def wrap_server(server_cert: LeafCert, raw_server_sock: socket.socket) -> ssl.SSLSocket:
+    def wrap_server(
+        server_cert: LeafCert, raw_server_sock: socket.socket
+    ) -> ssl.SSLSocket:
         ctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
         server_cert.configure_cert(ctx)
-        wrapped_server_sock = ctx.wrap_socket(
-            raw_server_sock, server_side=True)
+        wrapped_server_sock = ctx.wrap_socket(raw_server_sock, server_side=True)
         print("server encrypted with:", wrapped_server_sock.cipher())
         return wrapped_server_sock
 
@@ -362,12 +386,15 @@ def test_stdlib_end_to_end(key_type: KeyType) -> None:
 
 @pytest.mark.parametrize("key_type", [KeyType.RSA, KeyType.ECDSA])
 def test_pyopenssl_end_to_end(key_type: KeyType) -> None:
-    def wrap_client(ca: CA, raw_client_sock: socket.socket, hostname: str) -> OpenSSL.SSL.Connection:
+    def wrap_client(
+        ca: CA, raw_client_sock: socket.socket, hostname: str
+    ) -> OpenSSL.SSL.Connection:
         # Cribbed from example at
         #   https://service-identity.readthedocs.io/en/stable/api.html#service_identity.pyopenssl.verify_hostname
         ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
-        ctx.set_verify(OpenSSL.SSL.VERIFY_PEER,
-                       lambda conn, cert, errno, depth, ok: bool(ok))
+        ctx.set_verify(
+            OpenSSL.SSL.VERIFY_PEER, lambda conn, cert, errno, depth, ok: bool(ok)
+        )
         ca.configure_trust(ctx)
         conn = OpenSSL.SSL.Connection(ctx, raw_client_sock)
         conn.set_connect_state()
@@ -375,7 +402,9 @@ def test_pyopenssl_end_to_end(key_type: KeyType) -> None:
         service_identity.pyopenssl.verify_hostname(conn, hostname)
         return conn
 
-    def wrap_server(server_cert: LeafCert, raw_server_sock: socket.socket) -> OpenSSL.SSL.Connection:
+    def wrap_server(
+        server_cert: LeafCert, raw_server_sock: socket.socket
+    ) -> OpenSSL.SSL.Connection:
         ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
         server_cert.configure_cert(ctx)
 
@@ -397,38 +426,30 @@ def test_identity_variants() -> None:
     cases = {
         # Traditional ascii hostname
         "example.org": x509.DNSName("example.org"),
-
         # Wildcard
         "*.example.org": x509.DNSName("*.example.org"),
-
         # IDN
         "éxamplë.org": x509.DNSName("xn--xampl-9rat.org"),
         "xn--xampl-9rat.org": x509.DNSName("xn--xampl-9rat.org"),
-
         # IDN + wildcard
         "*.éxamplë.org": x509.DNSName("*.xn--xampl-9rat.org"),
         "*.xn--xampl-9rat.org": x509.DNSName("*.xn--xampl-9rat.org"),
-
         # IDN that acts differently in IDNA-2003 vs IDNA-2008
         "faß.de": x509.DNSName("xn--fa-hia.de"),
         "xn--fa-hia.de": x509.DNSName("xn--fa-hia.de"),
-
         # IDN with non-permissable character (uppercase K)
         # (example taken from idna package docs)
         "Königsgäßchen.de": x509.DNSName("xn--knigsgchen-b4a3dun.de"),
-
         # IP addresses
         "127.0.0.1": x509.IPAddress(IPv4Address("127.0.0.1")),
         "::1": x509.IPAddress(IPv6Address("::1")),
         # Check normalization
         "0000::1": x509.IPAddress(IPv6Address("::1")),
-
         # IP networks
         "127.0.0.0/24": x509.IPAddress(IPv4Network("127.0.0.0/24")),
         "2001::/16": x509.IPAddress(IPv6Network("2001::/16")),
         # Check normalization
         "2001:0000::/16": x509.IPAddress(IPv6Network("2001::/16")),
-
         # Email address
         "example@example.com": x509.RFC822Name("example@example.com"),
     }
@@ -440,9 +461,7 @@ def test_identity_variants() -> None:
         print(f"testing: {hostname!r}")
         pem = ca.issue_cert(hostname).cert_chain_pems[0].bytes()
         cert = x509.load_pem_x509_certificate(pem)
-        san = cert.extensions.get_extension_for_class(
-            x509.SubjectAlternativeName
-        )
+        san = cert.extensions.get_extension_for_class(x509.SubjectAlternativeName)
         assert_is_leaf(cert)
         got = list(san.value)[0]
         assert got == expected
@@ -464,27 +483,19 @@ def test_CN() -> None:
     # Default is no common name
     pem = ca.issue_cert("example.com").cert_chain_pems[0].bytes()
     cert = x509.load_pem_x509_certificate(pem)
-    common_names = cert.subject.get_attributes_for_oid(
-        x509.oid.NameOID.COMMON_NAME
-    )
+    common_names = cert.subject.get_attributes_for_oid(x509.oid.NameOID.COMMON_NAME)
     assert common_names == []
 
     # Common name on its own is valid
     pem = ca.issue_cert(common_name="woo").cert_chain_pems[0].bytes()
     cert = x509.load_pem_x509_certificate(pem)
-    common_names = cert.subject.get_attributes_for_oid(
-        x509.oid.NameOID.COMMON_NAME
-    )
+    common_names = cert.subject.get_attributes_for_oid(x509.oid.NameOID.COMMON_NAME)
     assert common_names[0].value == "woo"
 
     # Common name + SAN
     pem = ca.issue_cert("example.com", common_name="woo").cert_chain_pems[0].bytes()
     cert = x509.load_pem_x509_certificate(pem)
-    san = cert.extensions.get_extension_for_class(
-        x509.SubjectAlternativeName
-    )
+    san = cert.extensions.get_extension_for_class(x509.SubjectAlternativeName)
     assert list(san.value)[0] == x509.DNSName("example.com")
-    common_names = cert.subject.get_attributes_for_oid(
-        x509.oid.NameOID.COMMON_NAME
-    )
+    common_names = cert.subject.get_attributes_for_oid(x509.oid.NameOID.COMMON_NAME)
     assert common_names[0].value == "woo"
